package com.octoshape.android.octoandroidplayer;

// Android OSA
import octoshape.osa2.android.OctoshapeSystem;
import octoshape.osa2.android.StreamPlayer;
import octoshape.osa2.listeners.MultiStreamInfoListener;
import octoshape.osa2.listeners.OctoshapeSystemListener;
import octoshape.osa2.listeners.ProblemListener;
import octoshape.osa2.android.listeners.MediaPlayerListener;
import octoshape.osa2.android.listeners.StreamPlayerListener;
import octoshape.osa2.Problem;
import octoshape.osa2.listeners.StatusChangedListener;

// Android SDK
import com.octoshape.android.client.OctoStatic;




// Android Media Player 
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnErrorListener;
import android.media.MediaPlayer.OnVideoSizeChangedListener;
import android.media.MediaPlayer.OnInfoListener;
import android.media.MediaPlayer.OnPreparedListener;

// Android
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.Window;
import android.view.WindowManager;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;
import android.app.Activity;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;




// Java
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;

public class MiniAndroidPlayer extends Activity implements OnVideoSizeChangedListener, ProblemListener {

	private static final String LOGTAG = "OctoAndroidPlayer";
	private static final String OCTOLINK = "octoshape://streams.octoshape.net/demo/live/nba/abr";
    private static final boolean USE_AUTHENTICATON = false;
    private SurfaceView mSurface;
	private SurfaceHolder mHolder;
	private OctoshapeSystem os;
	private MediaPlayer mMediaPlayer;
	private StreamPlayer mStreamPlayer;
    private TextView mBitrate;
    private TextView mStatus;
    private TextView mUrl;

	/**
	 * Called when the activity is first created. Creating views, GUI, setting
	 * listeners. Here the OSA is initialized.
	 */
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().addFlags(android.view.WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
		setContentView(R.layout.video);
		initOctoshapeSystem();

		// Setup views and holder to be used by MediaPlayer
		mSurface = (SurfaceView) findViewById(R.id.surface);
        mBitrate = (TextView) findViewById(R.id.bitrate);
        mStatus = (TextView) findViewById(R.id.status);
        mUrl = (TextView) findViewById(R.id.url);
		mHolder = mSurface.getHolder();
		mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	}

	/**
	 * Creating OctoshapeSystem, adding ProblemListener and setting OctoshapePortListener triggering a
	 * callback once the Octoshape service/client has started successfully.
	 */
	public void initOctoshapeSystem() {

		// Create OctoshapeSystem
		os = OctoStatic.create(this, this, null);
		os.setOctoshapeSystemListener(new OctoshapeSystemListener() {
			// called once the Octoshape service/client has started.
			@Override
			public void onConnect(String authId) {
				mStreamPlayer = setupStream(OCTOLINK, authId);
				mStreamPlayer.requestPlay();
			}
		});
		// Adding AndroidMediaPlayer
		os.addPlayerNameAndVersion(OctoshapeSystem.MEDIA_PLAYER_NATIVE, OctoshapeSystem.MEDIA_PLAYER_NATIVE,"" + Build.VERSION.SDK_INT);
		os.open();
	}

	/**
	 * Adding a stream to the OctoMediaPlayer. Creates an StreamPlayer the
	 * StreamPlayer instance is initiated with it's own UrlListener and
	 * ProblemListener.
	 * 
	 * @param stream link (e.g., octoshape://streams.octoshape.net/demo/live/nba/abr)
	 * @param authId ID provided by the OctoshapeSystem on start-up.
     *
     * @return StreamPlayer instance allowing playback
	 */
	public StreamPlayer setupStream(final String stream, String authId) {

		Log.d(LOGTAG, "Setting up stream: " + stream);
		StreamPlayer sp = os.createStreamPlayer(stream);
		sp.setProblemListener(this);
        sp.setStatusChangedListener(new StatusChangedListener() {
            @Override
            public void statusChanged(byte oldStatus, final byte newStatus) {
                runOnUiThread(new Runnable() {
					@Override
					public void run() {
		            	mStatus.setText(StreamPlayer.getStatusString(newStatus));		
					}
				});
            	
            }
        });
        sp.setMultiStreamInfoListener(new MultiStreamInfoListener() {
            @Override
            public void gotMultiStreamInfo(String[] strings, int[] ints) {}
            @Override
            public void gotBaseStreamInfo(String s, final int i) {
                runOnUiThread(new Runnable() {
					@Override
					public void run() {
						mBitrate.setText((i/1000)+"k");
					}
                });
            }
        });
		sp.setListener(new StreamPlayerListener() {
			
			/**
			 * Receiving new URL from the streamplayer object either due to
			 * requesting playback, seeking or experiencing a bitrate/resolution
			 * changes requiring the re-initialization of the Player.
			 * 
			 * @param url
			 *            to be passed to the media player
			 * @param seekOffset
			 *            offset we have seek to in milliseconds
			 */
			@Override
			public void gotUrl(String url, long seekOffset, MediaPlayerListener mpl) {
				playStream(Uri.parse(url), mpl);
            }

			/**
			 * Resets an on-demand file duration previously reported in
			 * resolvedOsaSeek(..) method
			 */
			@Override
			public void gotNewOnDemandStreamDuration(long duration) {}

			/**
			 * Called if the stream is seekable using the media player's own
			 * native seeking functionality (e.g., the Android MediaPlayer does
			 * the seeking for us).
			 */
			@Override
			public void resolvedNativeSeek(boolean isLive, String playerId) {}

			/**
			 * Called if it is not possible to seek in the stream.
			 */
			@Override
			public void resolvedNoSeek(boolean isLive, String playerId) {}


			/**
			 * Called when stream support OsaSeek / DVR
			 */
            @Override
			public void resolvedOsaSeek(boolean isLive, long duration, String playerId) {}
        });
        if(USE_AUTHENTICATON) {
            try {
                authenticateStreamPlayer(sp, authId);
            } catch (IOException e) {
                Log.e(LOGTAG, "Error getting authentication token from server", e);
            }
        }
        return sp;
	}

    /** This is a reference implementation for the implementation token authentication
     *  see https://support.octoshape.com/entries/21368651-Implementing-Token-Authentication.
     *
     * @param streamPlayer stream to be authenticated
     * @param authId ID provided by the @ OctoshapeSystem on start-up.
     */
    private boolean authenticateStreamPlayer(StreamPlayer streamPlayer, String authId) throws IOException {

        final String ERROR = "ERROR_NO_AUTHID";
        URL url;

        // Simple Ticket
        if (authId == null)
            url = new URL("http://octoshape.com/broadcasters/webintegration/ticket/simple.php");

            // Stronger Security using Auth-ID
        else
            url = new URL(" http://octoshape.com/broadcasters/webintegration/ticket/authid.php?OctoshapeAuthid=" + authId);

        // Read server response
        BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
        String inputLine;
        ArrayList<String> response = new ArrayList<String>();
        while ((inputLine = in.readLine()) != null)
            response.add(inputLine);
        in.close();

        // Verifying server response and authenticate
        if (response.size() >= 2 && ERROR.equals(response.get(0))) {
            streamPlayer.setAuthorization(response.get(1), response.get(0));
            return true;
        } else {
            Log.e(LOGTAG, "Error stream authentication. Invalid server response.");
            return false;
        }
    }

	/**
	 * Setting up and playing a received media URL
	 * 
	 * @param mediaUrl URL which needs to be passed to a media player
	 * @param mpl MediaPlayerListen used for reporting stream start and end
	 */
	protected void playStream(final Uri mediaUrl, final MediaPlayerListener mpl) {

		Log.d(LOGTAG, "Now playing: " + mediaUrl);
		if (mMediaPlayer == null)
			mMediaPlayer = new MediaPlayer();
		
		mMediaPlayer.reset();
		try {
			mMediaPlayer.setDisplay(mHolder);
			mMediaPlayer.setDataSource(this, mediaUrl);
			mMediaPlayer.setOnVideoSizeChangedListener(this);
			mMediaPlayer.setOnCompletionListener(new OnCompletionListener() {
				@Override
				public void onCompletion(MediaPlayer mp) {
					mpl.onMediaPlaybackCompleted();
				}
			});
			if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.JELLY_BEAN_MR1)
				mMediaPlayer.setOnInfoListener(new OnInfoListener() {
					@Override
					public boolean onInfo(MediaPlayer mp, int what, int extra) {
						if (MediaPlayer.MEDIA_INFO_VIDEO_RENDERING_START == what) {
							Log.d("MediaPlayer", "START RENDERING");
							mpl.onMediaPlaybackStarted();
                            mUrl.setText(mediaUrl.toString());
                        }
						return false;
					}
				});
			if (android.os.Build.VERSION.SDK_INT < android.os.Build.VERSION_CODES.JELLY_BEAN_MR1)
				mMediaPlayer.setOnPreparedListener(new OnPreparedListener() {
					@Override
					public void onPrepared(MediaPlayer mp) {
						Log.d("MediaPlayer", "PLAYERBACK STARTED");
						mpl.onMediaPlaybackStarted();
                        mUrl.setText(mediaUrl.toString());

					}
				});
			mMediaPlayer.setOnErrorListener(new OnErrorListener() {
				@Override
				public boolean onError(MediaPlayer mp, int what, int extra) {
					MiniAndroidPlayer.this.gotProblem(OctoStatic.generateMediaPlayerProblem(what, extra));
					Log.d(LOGTAG, "MediaPlayer error:" + what + ":" + extra);
					return true;
				}
			});
			mMediaPlayer.prepare();
			mMediaPlayer.start();
		} catch (Exception e) {
			Log.e(LOGTAG, "Error preparing MediaPlayer", e);
			displayMessage("Error preparing MediaPlayer: " + e.getMessage());
		}
	}
	/**
	 * Handling onVideoSizeChanged called by the MediaPlayer after initiating
	 * playback, scales video according to aspect ratio and display dimensions.
	 */
	@Override
	public void onVideoSizeChanged(MediaPlayer mp, int width, int height) {

		RelativeLayout.LayoutParams vLayout = (RelativeLayout.LayoutParams) mSurface.getLayoutParams();
		vLayout.width = getWindowManager().getDefaultDisplay().getWidth();
		vLayout.height = getWindowManager().getDefaultDisplay().getHeight();

		float aspectRatio = (float) width / height;
		float screenRatio = (float) vLayout.width / vLayout.height;
		float topMargin = 0, leftMargin = 0;

		if (screenRatio < aspectRatio)
			topMargin = (float) vLayout.height - ((float) vLayout.width / aspectRatio);
		else if (screenRatio > aspectRatio)
			leftMargin = (float) vLayout.width - (vLayout.height * aspectRatio);

		vLayout.setMargins((int) leftMargin, (int) topMargin, 0, 0);
		mSurface.setLayoutParams(vLayout);
	}
	
	@Override
	public void gotProblem(final Problem p) {
		Log.e(LOGTAG, "Problem: "+p.toString());
		if(p.getMessage()!=null)
			displayMessage(p.getMessage() + "(" + p.getErrorCode() + ")");
	}
	
	private void displayMessage(final String error) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Toast.makeText(MiniAndroidPlayer.this, error, Toast.LENGTH_LONG).show();
			}
		});
	}

	@Override
	public void onBackPressed() {
		shutdown();
	}
	@Override
    public void onPause() {
		super.onPause();
		shutdown();
	}
	private void shutdown(){
		if (mMediaPlayer != null)
			mMediaPlayer.release();

		if( mStreamPlayer != null)
			mStreamPlayer.close(null);
		
		OctoStatic.terminate(new Runnable() {
			@Override
			public void run() {
				finish();
			}
		});
	}
}